@@ -1,16 +1,25 @@
-use super::*;
+use num_traits::real::Real;
 
 /// 2D projective transformations
 ///
 /// ![](https://s2.ax1x.com/2019/05/29/VuEtUA.png)
-#[doc = "# Examples"]
+///
+/// ## Supported 2D operations
+///
+/// | Operation                          | Description |
+/// |------------------------------------|-------------|
+/// | [transform](Projective::transform) | ----------- |
+/// | [translate](Projective::translate) | ----------- |
+/// | [rotate](Projective::rotate)       | ----------- |
+/// | [scale](Projective::scale)         | ----------- |
+#[doc = "## Examples"]
 #[doc = "```"]
 #[doc = include_str ! ("../../tests/point.rs")]
 #[doc = "```"]
 pub trait Projective<T>
 where
     Self: Sized,
-    T: Zero + One + Real,
+    T: Real,
 {
     /// Transform by a 3×3 matrix.
     ///
@@ -23,7 +32,57 @@ where
     ///     m_6 & m_7 & m_8 \\
     /// \end{bmatrix}
     /// ```
-    fn transform(&self, matrix: &[&T; 9]) -> Self;
+    fn transform(&mut self, matrix: &[&T; 9]);
+    /// Transform by length $\delta x, \delta y$.
+    ///
+    /// # Matrix
+    ///
+    /// ```math
+    /// \begin{bmatrix}
+    ///     1 & 0 & x \\
+    ///     0 & 1 & y \\
+    ///     0 & 0 & 1 \\
+    /// \end{bmatrix}
+    /// ```
+    #[inline(always)]
+    #[rustfmt::skip]
+    fn translate(&mut self, x: &T, y: &T) {
+        self.transform(&[
+            &T::one(),  &T::zero(), x,
+            &T::zero(), &T::one(),  y,
+            &T::zero(), &T::zero(), &T::one()
+        ])
+    }
+    /// Transform by length $\delta x$.
+    ///
+    /// # Matrix
+    ///
+    /// ```math
+    /// \begin{bmatrix}
+    ///     1 & 0 & x \\
+    ///     0 & 1 & 0 \\
+    ///     0 & 0 & 1 \\
+    /// \end{bmatrix}
+    /// ```
+    #[inline(always)]
+    fn translate_x(&mut self, x: &T) {
+        self.translate(x, &T::zero())
+    }
+    /// Transform by length $\delta y$.
+    ///
+    /// # Matrix
+    ///
+    /// ```math
+    /// \begin{bmatrix}
+    ///     1 & 0 & 0 \\
+    ///     0 & 1 & y \\
+    ///     0 & 0 & 1 \\
+    /// \end{bmatrix}
+    /// ```
+    #[inline(always)]
+    fn translate_y(&mut self, y: &T) {
+        self.translate(&T::zero(), y)
+    }
     /// Transform by rotate degree $\alpha$.
     ///
     /// # Matrix
@@ -35,8 +94,9 @@ where
     ///     0          & 0          & 1 \\
     /// \end{bmatrix}
     /// ```
+    #[inline(always)]
     #[rustfmt::skip]
-    fn rotate(&self, angle: &T) -> Self {
+    fn rotate(&mut self, angle: &T) {
         let cos = angle.cos();
         let sin = angle.sin();
         self.transform(&[
@@ -55,55 +115,11 @@ where
     ///     y =& (x_0 - p_x)\sin α + (y_0 - p_y)\cos α + p_y \\
     /// \end{aligned}
     /// ```
-    fn rotate_point(&self, point: &[&T; 2], angle: &T) -> Self {
-        self.translate(&point[0].neg(), &point[1].neg()).rotate(angle).translate(point[0], point[1])
-    }
-    /// Transform by length $\delta x, \delta y$.
-    ///
-    /// # Matrix
-    ///
-    /// ```math
-    /// \begin{bmatrix}
-    ///     1 & 0 & x \\
-    ///     0 & 1 & y \\
-    ///     0 & 0 & 1 \\
-    /// \end{bmatrix}
-    /// ```
-    #[rustfmt::skip]
-    fn translate(&self, x: &T, y: &T) -> Self {
-        self.transform(&[
-            &T::one(),  &T::zero(), x,
-            &T::zero(), &T::one(),  y,
-            &T::zero(), &T::zero(), &T::one(),
-        ])
-    }
-    /// Transform by length $\delta x$.
-    ///
-    /// # Matrix
-    ///
-    /// ```math
-    /// \begin{bmatrix}
-    ///     1 & 0 & x \\
-    ///     0 & 1 & 0 \\
-    ///     0 & 0 & 1 \\
-    /// \end{bmatrix}
-    /// ```
-    fn translate_x(&self, x: &T) -> Self {
-        self.translate(x, &T::zero())
-    }
-    /// Transform by length $\delta y$.
-    ///
-    /// # Matrix
-    ///
-    /// ```math
-    /// \begin{bmatrix}
-    ///     1 & 0 & 0 \\
-    ///     0 & 1 & y \\
-    ///     0 & 0 & 1 \\
-    /// \end{bmatrix}
-    /// ```
-    fn translate_y(&self, y: &T) -> Self {
-        self.translate(&T::zero(), y)
+    #[inline(always)]
+    fn rotate_point(&mut self, point: &[&T; 2], angle: &T) {
+        self.translate(&point[0].neg(), &point[1].neg());
+        self.rotate(angle);
+        self.translate(point[0], point[1])
     }
     /// Transform by scale $x, y$.
     ///
@@ -116,8 +132,9 @@ where
     ///     0 & 0 & 1 \\
     /// \end{bmatrix}
     /// ```
+    #[inline(always)]
     #[rustfmt::skip]
-    fn scale(&self, x: &T, y: &T) -> Self {
+    fn scale(&mut self, x: &T, y: &T)  {
         self.transform(&[
             x,          &T::zero(), &T::zero(),
             &T::zero(), y,          &T::zero(),
@@ -135,7 +152,8 @@ where
     ///     0 & 0 & 1 \\
     /// \end{bmatrix}
     /// ```
-    fn scale_x(&self, x: &T) -> Self {
+    #[inline(always)]
+    fn scale_x(&mut self, x: &T) {
         self.scale(x, &T::one())
     }
     /// Transform by scale $y$.
@@ -149,12 +167,14 @@ where
     ///     0 & 0 & 1 \\
     /// \end{bmatrix}
     /// ```
-    fn scale_y(&self, y: &T) -> Self {
+    #[inline(always)]
+    fn scale_y(&mut self, y: &T) {
         self.scale(&T::one(), y)
     }
     /// Transform by skew $x, y$.
+    #[inline(always)]
     #[rustfmt::skip]
-    fn reflect(&self, x: &T, y: &T) -> Self {
+    fn reflect(&mut self, x: &T, y: &T)  {
         self.transform(&[
             &T::one(),  &T::zero(), &T::zero(),
             &T::zero(), &T::one(),  &T::zero(),
@@ -162,11 +182,14 @@ where
         ])
     }
     /// Transform by skew $x$.
-    fn reflect_x(&self) -> Self {
+    #[inline(always)]
+    fn reflect_x(&mut self) {
         self.reflect(&T::zero(), &T::one())
     }
     /// Transform by skew $y$.
-    fn reflect_y(&self) -> Self {
+
+    #[inline(always)]
+    fn reflect_y(&mut self) {
         self.reflect(&T::one(), &T::zero())
     }
     /// Transform by angle $\alpha, \beta$.
@@ -180,8 +203,9 @@ where
     ///     0         & 0          & 1 \\
     /// \end{bmatrix}
     /// ```
+    #[inline(always)]
     #[rustfmt::skip]
-    fn skew(&self, a: &T, b: &T) -> Self {
+    fn skew(&mut self, a: &T, b: &T) {
         self.transform(&[
             &T::one(),  &a.tan(),   &T::zero(),
             &b.tan(),   &T::one(),  &T::zero(),
@@ -199,7 +223,8 @@ where
     ///     0         & 0          & 1 \\
     /// \end{bmatrix}
     /// ```
-    fn skew_x(&self, a: &T) -> Self {
+    #[inline(always)]
+    fn skew_x(&mut self, a: &T) {
         self.skew(a, &T::zero())
     }
     /// Transform by angle $\beta$.
@@ -213,7 +238,8 @@ where
     ///     0         & 0          & 1 \\
     /// \end{vmatrix}
     /// ```
-    fn skew_y(&self, b: &T) -> Self {
+    #[inline(always)]
+    fn skew_y(&mut self, b: &T) {
         self.skew(&T::zero(), b)
     }
 }
